/*******************************************************************
*
*                       Cinepak for SATURN Player 
*                                by SOJ
*                         usage sample program
*
* Using the file system to play a movie file on a CD
* Sample of playing a movie file on a CD in 16 million colors on VDP2 using the file system
* Movie file name is "SAMPLE.CPK"
*
* Define #define USE_STM if you want to use the stream system.
*
* Caution.
* Do not use Hint interrupt.
* V-blank In and Out interrupts should be as short as possible.
* Software transfer should be selected as the data transfer method.
*
*                      Copyright(c) 1994,1995 SEGA
*
*   Comment: main module
*   File   : SMPCPK6.c
*   Date   : 1994-10-31
*   Author : H.G
*
*******************************************************************/

/* Define this when using the stream system */
/* #define USE_STM */

/* Define this when you want the Cinepac library to use a slave CPU. */
/* #define DUAL_CPU */

/* Set the playback start trigger size. */
/* #define START_TRG_SIZE		 (RING_BUF_SIZ - 24*2048) */
/* #define START_TRG_SIZE		 (30 * 2048) */
#define START_TRG_SIZE		 (100 * 2048)

#include <machine.h>
#include <string.h>
#define _SH
#include "sega_xpt.h"
#include "sega_sys.h"
#include "sega_def.h"
#include "sega_mth.h"
#include "sega_scl.h" 
#include "sega_int.h"
#define  _SPR2_
#include "sega_spr.h"
#include "sega_dma.h"
#include "sega_tim.h"
#include "sega_cdc.h"
#include "sega_gfs.h"
#include "sega_stm.h"
#include "sega_per.h"
#include "sega_snd.h"
#include "sega_cpk.h"



void smpVblIn(void);
void smpVblOut(void);
void timeFunc(void);


/*------------------------- sMacro Constantst -------------------------*/

/* PCM RAM transfer address and number of samples */
#define	PCM_ADDR	((void*)0x25a20000)
#define	PCM_SIZE	(4096L*16)

/* Ring buffer size */
#define	RING_BUF_SIZ	(1024L*400)

/* TV screen size */
#define   DISP_XSIZE       320
#define   DISP_YSIZE       224

/* Minimum and maximum value of the volume */
#define LEVEL_MIN	(0)
#define LEVEL_MAX	(7)

/* Panning Min, Max, and Center */
#define PAN_MIN		(0)					/* Pan Left: maximum on the left, zero on the right */
#define PAN_MAX		(31)				/* Pan Right: right is maximum, left is zero */
#define PAN_CENTER	((PAN_MAX + 1) / 2)	/* Center: maximum left and right  	*/

/* 1/2 */
#define SMP_DIV2(a)		((a) >> 1)

/* Maximum image size that can be transferred in batches during V-blank */
#define SMPCPK_VBL_COPY_MAX			(288 * 144)

/*----------------------- sGlobal variablest -----------------------*/

/* Work Buffer */
Uint32 g_movie_work[CPK_24WORK_DSIZE];

/* Ring Buffer */
Uint32 g_movie_buf[RING_BUF_SIZ / sizeof(Uint32)];

/*  Image Decoding buffer */
Uint32 g_decode_buf[320L*240L*4/4];


/* Name of the file to play */
char *filename = "SAMPLE.CPK";

char *dirname = "MOV";

/* Movie Size */
Uint32			movie_x, movie_y;

/* Boolean for when a timmer interrupt is received. TRUE */
volatile Sint32 g_time_on;

/* If a V blank IN interrupt is received TRUE */
volatile Sint32 g_vbl_in;

/* Cycle pattern */
Uint16	cycle_tbl[]={
	0x4444,0x4444,
	0xffff,0xffff,
	0x4444,0x4444,
	0xffff,0xffff
};

/* Game Pad Info*/
Uint32	PadWorkArea[7];
Uint16	PadData1  = 0x0000;
Uint16	PadData1E = 0x0000;
Uint16	PadData2  = 0x0000;
Uint16	PadData2E = 0x0000;

/* Pause State */
volatile  Sint32		g_pause_at_once;
volatile  Sint32		g_pause_keyframe;

/* Volume Level */
Sint32 g_level = LEVEL_MAX;

/* Panning Value, Left = 0, Right = 31. */
Sint32 g_pan = PAN_CENTER;

/* Pan value: value specified as an argument */
Sint32 g_pan_tbl[PAN_MAX + 1] = {
	0x1f, 0x1e, 0x1d, 0x1c, 0x1b, 0x1a, 0x19, 0x18, 
	0x17, 0x16, 0x15, 0x14, 0x13, 0x12, 0x11, 0x10, 
	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 
	0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f
};

/*---------------------------- sFunctionst ----------------------------*/

/********************************************************************/
/* Function to be called when an error occurs.									*/
/********************************************************************/
void errGfsFunc(void *obj, Sint32 ec)
{
	/* Error Handling */
}

void errStmFunc(void *obj, Sint32 ec)
{
	/* Error Handling */
}

void errCpkFunc(void *obj, Sint32 ec)
{
	/* Error Handling */
}


/*====================== Screen Display Processing ===========================*/
 void memsetDword(Uint32 *dst, Uint32 value, Sint32 cnt)
{
	Uint32 *dst_stop = dst;

	dst	+= cnt;
	while (dst > dst_stop) {
		*--dst = value;
	}
}

 void dispSclInit(void)
{
/*	SclVramConfig	tp; */
	SclConfig		scfg;
	int				i;
	Uint16			BackCol;

	/* erase VDP2 */
	memsetDword((Uint32 *)SCL_VDP2_VRAM_A, 0x80000000, 512L * 512L/2);

   /*******************************************
    *	ucoQ Initialization                    *
    *******************************************/
	SCL_Vdp2Init();

	/* Register V-Blank Interrupt Routine. */
	INT_ChgMsk(INT_MSK_NULL,INT_MSK_VBLK_IN | INT_MSK_VBLK_OUT);

	/* Wait for V_Blank Out Interrupt */
	(*((volatile Uint32 *)0x25fe00a4)) &= 0xfffffffc;	/* First Clear */
	while( !((*((volatile Uint32 *)0x25fe00a4)) & 2) );
	PER_LInit(PER_KD_PERTIM,2,PER_SIZE_DGT,PadWorkArea,0);

	INT_SetScuFunc(INT_SCU_VBLK_IN, smpVblIn);
	INT_SetScuFunc(INT_SCU_VBLK_OUT, smpVblOut);
	INT_ChgMsk(INT_MSK_VBLK_IN | INT_MSK_VBLK_OUT,INT_MSK_NULL);

	/* H blank setting */
	SCL_SetFrameInterval(1);
	set_imask(0);

	/* Timer 0 interrupt setting */
	INT_SetScuFunc(INT_SCU_TIM0, timeFunc);

   /*******************************************
    *	Scroll configuration settings                *
    *******************************************/
	SCL_InitConfigTb(&scfg);
	scfg.dispenbl  = ON;
	scfg.platesize = SCL_PL_SIZE_1X1;
	scfg.bmpsize   = SCL_BMP_SIZE_512X256;
	scfg.coltype   = SCL_COL_TYPE_1M;
	scfg.datatype  = SCL_BITMAP;
	scfg.mapover   = SCL_OVER_2;
	for(i=0;i<16;i++)
		scfg.plate_addr[i] = SCL_VDP2_VRAM_A;
	SCL_SetConfig(SCL_NBG0, &scfg);

   /*******************************************
    *	Set priority (0 to 7)                 *
    *******************************************/
	SCL_SetPriority(SCL_SPR,0);
	SCL_SetPriority(SCL_NBG0,7);

	/* Set the cycle pattern */
    SCL_SetCycleTable(cycle_tbl);

   /*******************************************
    *	Set the back screen color to black.              *
    *******************************************/
	BackCol = 0x0000;
	SCL_SetBack(SCL_VDP2_VRAM+0x80000-2,1,&BackCol);

	SCL_Open(SCL_NBG0);
		SCL_MoveTo(FIXED(0), FIXED(0),0);/* Home Position */
		SCL_Scale(FIXED(1.0), FIXED(1.0));
		SCL_SetRotateViewPoint(160,112,500);
		SCL_SetRotateCenter(160,112,0);
	SCL_Close();
	SCL_DisplayFrame();
}

/*====================== V-blank processing ===========================*/
 void smpVblIn(void)
{

	/* Call Cinepak's VblIn routine */
	CPK_VblIn();

	/* In order to use the graphics library, you have to run it. */
	SCL_VblankStart();
	g_vbl_in = TRUE;

}


 void smpVblOut(void)
{
	PerDgtInfo	*pad;
	PerDgtInfo	*mul_info;

	PER_LGetPer((PerGetPer **)&pad, (PerMulInfo **)&mul_info);
	PadData1  = pad[0].data ^ 0xffff;
	PadData1E |= pad[0].push ^ 0xffff;
	PadData2  = pad[1].data ^ 0xffff;
	PadData2E |= pad[1].push ^ 0xffff;

	/* In order to use the graphics library, you have to run it. */
	SCL_VblankEnd();
}

/* Timer 0 interrupt */
 void timeFunc(void)
{
	g_time_on = TRUE;
}


/*  RAM cycle pattern (bank A0) register */
#define		CYCLE_A_REG		0x25f80010

/*  VRAM cycle pattern (bank B0) register */
#define		CYCLE_B_REG		0x25f80018

/* Turn on CPU read/write mode */
#define		CYCLE_CPU_WRITE(reg_addr)	\
	{ \
	*((Uint32 *)(reg_addr)) = 0xeeeeeeee; \
	}

/* Set to character pattern data read mode */
#define		CYCLE_VDP_READ(reg_addr) \
	{ \
	*((Uint32 *)(reg_addr)) = 0x44444444; \
	}


#define	DMANOWAIT	0 /* 0 no wait */
/* 2-split transfer (a lot of wait, but it's not an option if the image size is large) */
 void copyDma2(Uint32 *src, Uint32 *dst, int movie_x, int movie_y)
{
	Sint32		copy_size = 4 * movie_x;
	Uint32 		*src_stop1 = src + movie_x * SMP_DIV2(movie_y);
	Uint32 		*src_stop2 = src + movie_x * movie_y;

	/* Top half transfer */
	g_time_on = FALSE;
	while (g_time_on == FALSE) ;
	g_vbl_in = FALSE;

	/* Set bank A0 to CPU write mode */
	CYCLE_CPU_WRITE(CYCLE_A_REG);

	while (src < src_stop1) {
		DMA_ScuMemCopy(dst, src, copy_size);
		src += movie_x;
		dst += SCL_MAXLINE;
#if DMANOWAIT
		while (DMA_ScuResult() == DMA_SCU_BUSY) ;
#endif
	}
	/* Make bank A0 a character pattern data read. */
	CYCLE_VDP_READ(CYCLE_A_REG);

	/* Bottom half transfer  */
	while (g_vbl_in == FALSE);

	/* Set bank B0 to CPU write mode */
	CYCLE_CPU_WRITE(CYCLE_B_REG);

	while (src < src_stop2) {
		DMA_ScuMemCopy(dst, src, copy_size);
		src += movie_x;
		dst += SCL_MAXLINE;
#if DMANOWAIT
		while (DMA_ScuResult() == DMA_SCU_BUSY) ;
#endif
	}
	/* Make bank B0 a character pattern data read. */
	CYCLE_VDP_READ(CYCLE_B_REG);
}

#if 0
/* 2-split transfer (VblIn starts the transfer of the upper half, followed by the transfer of the lower half) */
 void copyDma2(Uint32 *src, Uint32 *dst, int movie_x, int movie_y)
{
	Sint32		copy_size = 4 * movie_x;
	Uint32 		*src_stop1 = src + movie_x * SMP_DIV2(movie_y);
	Uint32 		*src_stop2 = src + movie_x * movie_y;


	/* Top half transfer */
	g_vbl_in = FALSE;
	while (g_vbl_in == FALSE);


	/* Set bank A0 to CPU write mode */
	CYCLE_CPU_WRITE(CYCLE_A_REG);

	while (src < src_stop1) {
		DMA_ScuMemCopy(dst, src, copy_size);
		src += movie_x;
		dst += SCL_MAXLINE;
#if DMANOWAIT
		while (DMA_ScuResult() == DMA_SCU_BUSY) ;
#endif
	}
	
	/* Make bank A0 a character pattern data read. */
	CYCLE_VDP_READ(CYCLE_A_REG);


	/* Bottom half transfer */
	/* Don't wait, keep transferring. */

	/* Set bank B0 to CPU write mode */
	CYCLE_CPU_WRITE(CYCLE_B_REG);

	while (src < src_stop2) {
		DMA_ScuMemCopy(dst, src, copy_size);
		src += movie_x;
		dst += SCL_MAXLINE;
#if DMANOWAIT
		while (DMA_ScuResult() == DMA_SCU_BUSY) ;
#endif
	}
	/* Make bank B0 a character pattern data read. */
	CYCLE_VDP_READ(CYCLE_B_REG);

}
#endif

/* Batch transfer (if the size can be transferred during Vbl) */
 void copyDma1(Uint32 *src, Uint32 *dst, int movie_x, int movie_y)
{
	Sint32		copy_size = 4 * movie_x;
	Uint32 		*src_stop1 = src + movie_x * SMP_DIV2(movie_y);
	Uint32 		*src_stop2 = src + movie_x * movie_y;

	/* Top half transfer */
	g_time_on = FALSE;
	while (g_time_on == FALSE);
	g_vbl_in = FALSE;

	/* Set banks A0 and B0 to CPU write mode */
	/* Set bank A0 to CPU write mode */
	CYCLE_CPU_WRITE(CYCLE_A_REG);

	while (src < src_stop1) {
		DMA_ScuMemCopy(dst, src, copy_size);
		src += movie_x;
		dst += SCL_MAXLINE;
#if DMANOWAIT
		while (DMA_ScuResult() == DMA_SCU_BUSY) ;
#endif
	}

	/* Make bank A0 a character pattern data read. */
	CYCLE_VDP_READ(CYCLE_A_REG);

	/* Bottom half transfer */
	/*Don't wait, transfer all at once during Vbl. */
	/* while (g_vbl_in == FALSE); */

	/* Set bank B0 to CPU write mode */
	CYCLE_CPU_WRITE(CYCLE_B_REG);

	while (src < src_stop2) {
		DMA_ScuMemCopy(dst, src, copy_size);
		src += movie_x;
		dst += SCL_MAXLINE;
#if DMANOWAIT
		while (DMA_ScuResult() == DMA_SCU_BUSY) ;
#endif
	}
	/* Make bank B0 a character pattern data read. */
	CYCLE_VDP_READ(CYCLE_B_REG);
}

 void copyDma(Uint32 *src, Uint32 *dst, int movie_x, int movie_y)
{
	if (movie_x * movie_y <= SMPCPK_VBL_COPY_MAX) {

		/* Batch transfer (if the size can be transferred during Vbl) */
		copyDma1(src, dst, movie_x, movie_y);
	} else {

		/* 2-split transfer (a lot of wait, but it's not an option if the image size is large) */
		copyDma2(src, dst, movie_x, movie_y);
	}
}


/*====================== Sound Processing ===========================*/
#define SDDRVS_TSK_SIZE			(0x6000)
#define BOOTSND_MAP_SIZE		(0x0100)
Sint32 sddrvs_tsk[SDDRVS_TSK_SIZE / 4];
Sint32 bootsnd_map[BOOTSND_MAP_SIZE / 4];

 Sint32 fileLoad(Sint8 *name, void *addr, Sint32 bsize)
{
	Sint32 		fid, i;

	for (i = 0; i < 10; i++) {
		fid = GFS_NameToId(name);
		if (fid >= 0) {
			GFS_Load(fid, 0, addr, bsize);
			return 0;
		}
	}
	return -1;
}

 void sndInit(void)
{
	SndIniDt 	snd_init;

	if (fileLoad("SDDRVS.TSK", (void *)sddrvs_tsk, SDDRVS_TSK_SIZE)) {
		return;
	}
	if (fileLoad("BOOTSND.MAP", (void *)bootsnd_map, BOOTSND_MAP_SIZE)) {
		return;
	}
	SND_INI_PRG_ADR(snd_init) 	= (Uint16 *)sddrvs_tsk;
	SND_INI_PRG_SZ(snd_init) 	= (Uint16 )SDDRVS_TSK_SIZE;
	SND_INI_ARA_ADR(snd_init) 	= (Uint16 *)bootsnd_map;
	SND_INI_ARA_SZ(snd_init) 	= (Uint16)BOOTSND_MAP_SIZE;
	SND_Init(&snd_init);
/*	SND_ChgMap(0); */
}


/*====================== File Processing ===========================*/

/* Maximum number of files in the root directory */
#define MAX_DIR		50

/* Maximum number of files to open simultaneously */
#define OPEN_MAX	5

/* ID of the stream group */
 StmGrpHn grp_hd;

/* Directory Information Management Domain */
 GfsDirTbl	dir_tbl;

/* Information containing the file name */
 GfsDirName dir_name[MAX_DIR];

/* GFS workspace */
 Uint8 gfs_work[GFS_WORK_SIZE(OPEN_MAX)];

	Uint8   stm_work[STM_WORK_SIZE(12, 24)];

 void fileInit(void)
{
	Sint32 file_num;

    /* Initialize the file system */
	GFS_DIRTBL_TYPE(&dir_tbl) = GFS_DIR_NAME;
	GFS_DIRTBL_DIRNAME(&dir_tbl) = dir_name;
	GFS_DIRTBL_NDIR(&dir_tbl) = MAX_DIR;
    file_num = GFS_Init(OPEN_MAX, gfs_work, &dir_tbl);
	if (file_num < 0) {
		return;
	}

	/* Setting the Error Function */
	GFS_SetErrFunc(errGfsFunc, NULL);

#ifdef USE_STM
	/* Initializing the stream system */
	STM_Init(12, 24, stm_work);

	/* Setting the Error Function */
	STM_SetErrFunc(errStmFunc, NULL);
#endif

}

 void fileInit2(void)
{
/*	Sint32 file_num; */
/*	changeDir(dirname); */

#ifdef USE_STM
	/* Stream Group Open */
	grp_hd = STM_OpenGrp();
	if (grp_hd == NULL) {
		return;
	}
	STM_SetExecGrp(grp_hd);
#endif
}

 StmHn stmOpen(char *fname)
{
    Sint32 fid;
	StmKey key;

    /* Find the file identifier from the file name */
    fid = GFS_NameToId(fname);
	STM_KEY_FN(&key) = STM_KEY_CN(&key) = STM_KEY_SMMSK(&key) = 
		STM_KEY_SMVAL(&key) = STM_KEY_CIMSK(&key) = STM_KEY_CIVAL(&key) =
		STM_KEY_NONE;
	return STM_OpenFid(grp_hd, fid, &key, STM_LOOP_NOREAD);
}

 void stmClose(StmHn fp)
{
	STM_Close(fp);
}

 	GfsDirTbl	dir_tbl2;
GfsDirName dir_name2[MAX_DIR];

 void changeDir(char *dir)
{
	Sint32	fid;

	fid = GFS_NameToId(dir);

	GFS_DIRTBL_TYPE(&dir_tbl2) = GFS_DIR_NAME;
	GFS_DIRTBL_DIRNAME(&dir_tbl2) = dir_name2;
	GFS_DIRTBL_NDIR(&dir_tbl2) = MAX_DIR;
	GFS_LoadDir(fid, &dir_tbl2);
	GFS_SetDir(&dir_tbl2);
}

 GfsHn fileOpen(char *fname)
{
    Sint32 fid;

    /* Find the file identifier from the file name */
    fid = GFS_NameToId(fname);
    return GFS_Open(fid);
}

 void fileClose(GfsHn fp)
{
	GFS_Close(fp);
}

/* Cinepak Initialization */
void cpkInit(void)
{
	/* Initialize Cinepak */
	CPK_Init();

	/* Setting the Error Function */
	CPK_SetErrFunc(errCpkFunc, NULL);

#ifdef DUAL_CPU
	/* Let the slave CPU be used by the Cinepac library. */
	CPK_SetCpu(CPK_CPU_DUAL);
#endif
}

/* Generate movie handle */
 CpkHn createMovie(
	#ifdef USE_STM
		StmHn stm
	#else	
		GfsHn gfs
	#endif
)
{
	CpkCreatePara	para;
	CpkHeader		*header;
	CpkHn			cpk;

	/* Generate movie handle */
	CPK_PARA_WORK_ADDR(&para) = g_movie_work;
	CPK_PARA_WORK_SIZE(&para) = CPK_24WORK_BSIZE;
	CPK_PARA_BUF_ADDR(&para) = g_movie_buf;
	CPK_PARA_BUF_SIZE(&para) = RING_BUF_SIZ;
	CPK_PARA_PCM_ADDR(&para) = PCM_ADDR;
	CPK_PARA_PCM_SIZE(&para) = PCM_SIZE;
#ifdef USE_STM
	cpk = CPK_CreateStmMovie(&para, stm);
#else
	cpk = CPK_CreateGfsMovie(&para, gfs);
#endif
	if (cpk == NULL) {
		return NULL;
	}

	/* Set the transfer method to Software Transfer. */
	CPK_SetTrModePcm(cpk, CPK_TRMODE_CPU); 
/*	CPK_SetTrModePcm(cpk, CPK_TRMODE_SDMA); */
/*	CPK_SetTrModeCd(cpk, CPK_TRMODE_CPU); */
/*	CPK_SetTrModeCd(cpk, CPK_TRMODE_SDMA); */

	CPK_SetTrModeCd(cpk, CPK_TRMODE_SCU); 
	CPK_SetLoadNum(cpk, 10);

	/* Set the number of display colors to 16 million. */
	CPK_SetColor(cpk, CPK_COLOR_24BIT);

	/* Load the header */
	/* If the size of the movie is known in advance, there is no need to call CPK_Preload and CPK_GetHeader. */
	/* In the case of a file system, it is better not to call CPK_PreloadHeader() to start playback a little earlier. */
	CPK_PreloadHeader(cpk);

	/* Get the size of the Movie */
	header = CPK_GetHeader(cpk);
	movie_x = header->width;
	movie_y = header->height;

	/* Set the Decoding Address of the movie */
	CPK_SetDecodeAddr(cpk, (void *)g_decode_buf, movie_x * 4);

#ifdef START_TRG_SIZE
	/* Set the playback start trigger size [byte] */
	CPK_SetStartTrgSize(cpk, START_TRG_SIZE);
#endif

	CPK_SetVolume(cpk, g_level);
	CPK_SetPan(cpk, g_pan_tbl[g_pan]);

	return cpk;
}

void smp_PadCtrlCpk(CpkHn cpk, Uint16 PadData1EW, Uint16 PadData1W)
{
/*	Sint32 i; */

	/* Pause */
	if (PadData1EW & PER_DGT_S) {
		PadData1EW = 0;
		if (g_pause_at_once == FALSE) {
			/* Start */
			g_pause_at_once = TRUE;
			CPK_Pause(cpk, CPK_PAUSE_ON_AT_ONCE);
		} else {
			/* Resume */
			g_pause_at_once = FALSE;
			CPK_Pause(cpk, CPK_PAUSE_OFF);
		}
	}
	if (PadData1EW & PER_DGT_C) {
		PadData1EW = 0;
		if (g_pause_keyframe == FALSE) {
			/* Start */
			g_pause_keyframe = TRUE;
			CPK_Pause(cpk, CPK_PAUSE_ON_KEYFRAME);
		} else {
			/* Resume */
			g_pause_keyframe = FALSE;
			CPK_Pause(cpk, CPK_PAUSE_OFF);
		}
	}

	/* Stop */
	if (PadData1EW & PER_DGT_A) {
		PadData1EW = 0;
		CPK_Stop(cpk);
	}

#if 0
	/* Change */
	if ((g_change_flag == FALSE) && (PadData1EW & PER_DGT_A)) {
		PadData1EW = 0;
		g_change_flag = TRUE;
		goNextStm(stm[1]);
	}
	/* Force switching of movie */
	if (g_change_flag == TRUE) {
		if (CPK_CheckChange() == CPK_CHANGE_OK_AT_ONCE) {
			CPK_Change();
			g_change_flag = FALSE;
		}
	}
#endif

	/* Increase Volume */
	if (PadData1EW & PER_DGT_U) {
		PadData1EW = 0;
		g_level = MIN(g_level + 1, LEVEL_MAX);
		CPK_SetVolume(cpk, g_level);
	}
	/* Decrease Volume */
	if (PadData1EW & PER_DGT_D) {
		PadData1EW = 0;
		g_level = MAX(g_level - 1, LEVEL_MIN);
		CPK_SetVolume(cpk, g_level);
	}

	/* Pan, from right (right maximum, left gradually weaker) */
	if (PadData1EW & PER_DGT_R) {
		PadData1EW = 0;
		g_pan = MIN(g_pan + 1, PAN_MAX);
		CPK_SetPan(cpk, g_pan_tbl[g_pan]);
	}
	/* Pan, from the left (maximum on the left, gradually decreasing on the right) */
	if (PadData1EW & PER_DGT_L) {
		PadData1EW = 0;
		g_pan = MAX(g_pan - 1, PAN_MIN);
		CPK_SetPan(cpk, g_pan_tbl[g_pan]);
	}
}

#define SMP_PAD_MODE_NUM		(1)
Sint32 smp_pad_mode = 0;

/* Pad controls */
void smp_PadCtrl(CpkHn cpk)
{
	Uint16   PadData1EW;
	Uint16   PadData1W;

	PadData1EW = PadData1E;
	PadData1E = 0;
	PadData1W = PadData1;
	PadData1 = 0;

	/* Switch pad processing targets */
	if (PadData1EW & PER_DGT_TL) {
		PadData1EW = 0;
		smp_pad_mode++;
	}

	switch (smp_pad_mode % SMP_PAD_MODE_NUM) {
	case 0:
		/* Pad control: playback library */
		smp_PadCtrlCpk(cpk, PadData1EW, PadData1W);
		break;
	}
}

void main(void)
{
#ifdef USE_STM
	StmHn			stm;
#else
	GfsHn			gfs;
#endif
	CpkHn 			cpk;
	Uint32			restart;
	Uint32			movie_lx, movie_ly;
	Uint32			*vram_addr;
	char 			*fname;

	/* Initialize variables */
	g_time_on = FALSE;
	g_vbl_in = FALSE;

	/* Scroll settings */
	dispSclInit();

	/* File Initialization */
	fileInit();

	/* Sound Initialization */
	sndInit();

	/* DMA Initialization */
	DMA_ScuInit();

	/* Cinepak Initialization */
	cpkInit();

	/* File Initialization 2 */
	fileInit2();

	/* Timer 0 settings */
	TIM_T0_SET_CMP(DISP_YSIZE/2 + 2);
	TIM_T1_SET_MODE(0x101);
	TIM_T0_ENABLE();

	restart = 1;

	while (1) {
		if (restart) {

			/* erase VDP2 */
			memsetDword((Uint32 *)SCL_VDP2_VRAM_A, 0x80000000, 512L * 512L/2);

			fname = filename;

#ifdef USE_STM
			/* Open Stream */
			if ((stm = stmOpen(fname)) == NULL) {
				continue;
			}
			/* Generate movie handle */
			cpk = createMovie(stm);
#else
			/* Open File */
			if ((gfs = fileOpen(fname)) == NULL) {
				continue;
			}
			/* Generate movie handle */
			cpk = createMovie(gfs);
#endif

			/* Determine Display Position */
			movie_lx = (DISP_XSIZE - movie_x)/2;
			movie_ly = (DISP_YSIZE - movie_y)/2;
			vram_addr = (Uint32 *)(SCL_VDP2_VRAM + 
						4 * (SCL_MAXLINE * movie_ly + movie_lx)
						+ 4 * SCL_MAXLINE * (SCL_MAXLINE/2 - DISP_YSIZE) / 2);

			/* Screen Scroll settings */
			SCL_Open(SCL_NBG0);
			SCL_MoveTo(FIXED(0), FIXED((SCL_MAXLINE/2 - DISP_YSIZE) / 2), 0);
			SCL_Close();
			SCL_DisplayFrame();

			/* Movie Start */
			CPK_Start(cpk);

			restart = 0;
		}

		/* Movie playback process */
		CPK_Task(cpk);

		/* Check for screen display requests */
		if (CPK_IsDispTime(cpk) == TRUE) {

			/* Image Transfer */
			copyDma(g_decode_buf, vram_addr, movie_x, movie_y);

			CPK_CompleteDisp(cpk);

		}
#if 0
		/* Pad controls */
		smp_PadCtrl(cpk);
#endif
		/* Movie End */
		if (CPK_GetPlayStatus(cpk) == CPK_STAT_PLAY_END) {

#ifdef USE_STM
			/* Destroy Movie */
			CPK_DestroyStmMovie(cpk);
			stmClose(stm);
#else
			/* Destroy Movie */
			CPK_DestroyGfsMovie(cpk);
			fileClose(gfs);
#endif
			restart = 1;
		}
	}
}
